

# Module pooler #
* [Description](#description)
* [Data Types](#types)
* [Function Index](#index)
* [Function Details](#functions)

This is the main interface to the pooler application.

Copyright (c) 2011-2013 Seth Falcon

__Behaviours:__ [`gen_server`](gen_server.md).

__Authors:__ Seth Falcon ([`seth@userprimary.net`](mailto:seth@userprimary.net)).

<a name="description"></a>

## Description ##
To integrate with your application, you probably want to call
`application:start(pooler)` after having specified appropriate
configuration for the pooler application (either via a config file
or appropriate calls to the application module to set the
application's config).

<a name="types"></a>

## Data Types ##




### <a name="type-consumers_map">consumers_map()</a> ###


<pre><code>
consumers_map() = #{pid() =&gt; {reference(), [pid()]}}
</code></pre>

 Internal



### <a name="type-gen_server_from">gen_server_from()</a> ###


<pre><code>
gen_server_from() = <a href="/home/seriy/workspace/stdlib/doc/gen_server.md#type-from">gen_server:from()</a>
</code></pre>




### <a name="type-gen_server_from">gen_server_from()</a> ###


<pre><code>
gen_server_from() = <a href="/home/seriy/workspace/stdlib/doc/gen_server.md#type-from">gen_server:from()</a>
</code></pre>




### <a name="type-group_name">group_name()</a> ###


<pre><code>
group_name() = atom()
</code></pre>

 The name of the group pool belongs to



### <a name="type-member_info">member_info()</a> ###


<pre><code>
member_info() = {reference(), free | pid(), <a href="/home/seriy/workspace/erts/doc/erlang.md#type-timestamp">erlang:timestamp()</a>}
</code></pre>




### <a name="type-members_map">members_map()</a> ###


<pre><code>
members_map() = #{pid() =&gt; <a href="#type-member_info">member_info()</a>}
</code></pre>

 Internal



### <a name="type-pool_config">pool_config()</a> ###


<pre><code>
pool_config() = [{name, <a href="#type-pool_name">pool_name()</a>} | {init_count, non_neg_integer()} | {max_count, non_neg_integer()} | {start_mfa, {module(), atom(), [any()]}} | {group, <a href="#type-group_name">group_name()</a>} | {cull_interval, <a href="#type-time_spec">time_spec()</a>} | {max_age, <a href="#type-time_spec">time_spec()</a>} | {member_start_timeout, <a href="#type-time_spec">time_spec()</a>} | {queue_max, non_neg_integer()} | {metrics_api, folsom | exometer} | {metrics_mod, module()} | {stop_mfa, {module(), atom(), [$pooler_pid | any(), ...]}} | {auto_grow_threshold, non_neg_integer()} | {add_member_retry, non_neg_integer()}]
</code></pre>




### <a name="type-pool_name">pool_name()</a> ###


<pre><code>
pool_name() = atom()
</code></pre>

 The name of the pool



### <a name="type-requestor_queue">requestor_queue()</a> ###


<pre><code>
requestor_queue() = <a href="/home/seriy/workspace/stdlib/doc/queue.md#type-queue">queue:queue</a>({<a href="#type-gen_server_from">gen_server_from()</a>, reference()})
</code></pre>

 Internal



### <a name="type-time_spec">time_spec()</a> ###


<pre><code>
time_spec() = {non_neg_integer(), <a href="#type-time_unit">time_unit()</a>}
</code></pre>

 Human-friendly way to specify the amount of time



### <a name="type-time_unit">time_unit()</a> ###


<pre><code>
time_unit() = min | sec | ms | mu
</code></pre>

<a name="index"></a>

## Function Index ##


<table width="100%" border="1" cellspacing="0" cellpadding="2" summary="function index"><tr><td valign="top"><a href="#accept_member-2">accept_member/2</a></td><td>For INTERNAL use.</td></tr><tr><td valign="top"><a href="#call_free_members-2">call_free_members/2</a></td><td>Invokes <code>Fun</code> with arity 1 over all free members in pool with <code>PoolName</code>.</td></tr><tr><td valign="top"><a href="#call_free_members-3">call_free_members/3</a></td><td>Invokes <code>Fun</code> with arity 1 over all free members in pool with <code>PoolName</code>.</td></tr><tr><td valign="top"><a href="#code_change-3">code_change/3</a></td><td></td></tr><tr><td valign="top"><a href="#group_pools-1">group_pools/1</a></td><td>Obtain the pids of all pools which are members of the group.</td></tr><tr><td valign="top"><a href="#handle_call-3">handle_call/3</a></td><td></td></tr><tr><td valign="top"><a href="#handle_cast-2">handle_cast/2</a></td><td></td></tr><tr><td valign="top"><a href="#handle_continue-2">handle_continue/2</a></td><td></td></tr><tr><td valign="top"><a href="#handle_info-2">handle_info/2</a></td><td></td></tr><tr><td valign="top"><a href="#init-1">init/1</a></td><td></td></tr><tr><td valign="top"><a href="#manual_start-0">manual_start/0</a></td><td></td></tr><tr><td valign="top"><a href="#new_pool-1">new_pool/1</a></td><td>Start a new pool described by the proplist <code>PoolConfig</code>.</td></tr><tr><td valign="top"><a href="#pool_child_spec-1">pool_child_spec/1</a></td><td>Get child spec described by the proplist <code>PoolConfig</code>.</td></tr><tr><td valign="top"><a href="#pool_stats-1">pool_stats/1</a></td><td>Obtain runtime state info for all workers.</td></tr><tr><td valign="top"><a href="#pool_utilization-1">pool_utilization/1</a></td><td>Obtain utilization info for a pool.</td></tr><tr><td valign="top"><a href="#return_group_member-2">return_group_member/2</a></td><td>Return a member that was taken from the group
<code>GroupName</code>.</td></tr><tr><td valign="top"><a href="#return_group_member-3">return_group_member/3</a></td><td>Return a member that was taken from the group <code>GroupName</code>.</td></tr><tr><td valign="top"><a href="#return_member-2">return_member/2</a></td><td>Return a member to the pool so it can be reused.</td></tr><tr><td valign="top"><a href="#return_member-3">return_member/3</a></td><td>Return a member to the pool so it can be reused.</td></tr><tr><td valign="top"><a href="#rm_group-1">rm_group/1</a></td><td>Terminates the group and all pools in that group.</td></tr><tr><td valign="top"><a href="#rm_pool-1">rm_pool/1</a></td><td>Terminate the named pool.</td></tr><tr><td valign="top"><a href="#start-0">start/0</a></td><td></td></tr><tr><td valign="top"><a href="#start_link-1">start_link/1</a></td><td></td></tr><tr><td valign="top"><a href="#stop-0">stop/0</a></td><td></td></tr><tr><td valign="top"><a href="#take_group_member-1">take_group_member/1</a></td><td>Take a member from a randomly selected member of the group
<code>GroupName</code>.</td></tr><tr><td valign="top"><a href="#take_member-1">take_member/1</a></td><td>Obtain exclusive access to a member from <code>PoolName</code>.</td></tr><tr><td valign="top"><a href="#take_member-2">take_member/2</a></td><td>Obtain exclusive access to a member of 'PoolName'.</td></tr><tr><td valign="top"><a href="#terminate-2">terminate/2</a></td><td></td></tr></table>


<a name="functions"></a>

## Function Details ##

<a name="accept_member-2"></a>

### accept_member/2 ###

<pre><code>
accept_member(PoolName::atom() | pid(), MemberPid::pid() | {noproc, term()}) -&gt; ok
</code></pre>
<br />

For INTERNAL use. Adds `MemberPid` to the pool.

<a name="call_free_members-2"></a>

### call_free_members/2 ###

<pre><code>
call_free_members(PoolName::<a href="#type-pool_name">pool_name()</a> | pid(), Fun::fun((pid()) -&gt; term())) -&gt; Res
</code></pre>

<ul class="definitions"><li><code>Res = [{ok, term()} | {error, term()}]</code></li></ul>

Invokes `Fun` with arity 1 over all free members in pool with `PoolName`.

<a name="call_free_members-3"></a>

### call_free_members/3 ###

<pre><code>
call_free_members(PoolName::<a href="#type-pool_name">pool_name()</a> | pid(), Fun, Timeout::timeout()) -&gt; Res
</code></pre>

<ul class="definitions"><li><code>Fun = fun((pid()) -&gt; term())</code></li><li><code>Res = [{ok, term()} | {error, term()}]</code></li></ul>

Invokes `Fun` with arity 1 over all free members in pool with `PoolName`.
`Timeout` sets the timeout of gen_server call.

<a name="code_change-3"></a>

### code_change/3 ###

<pre><code>
code_change(OldVsn::term(), State::term(), Extra::term()) -&gt; {ok, term()}
</code></pre>
<br />

<a name="group_pools-1"></a>

### group_pools/1 ###

<pre><code>
group_pools(GroupName::<a href="#type-group_name">group_name()</a>) -&gt; [pid()]
</code></pre>
<br />

Obtain the pids of all pools which are members of the group.

<a name="handle_call-3"></a>

### handle_call/3 ###

`handle_call(Request, From, Pool) -> any()`

<a name="handle_cast-2"></a>

### handle_cast/2 ###

<pre><code>
handle_cast(Msg::term(), Pool::term()) -&gt; {noreply, term()}
</code></pre>
<br />

<a name="handle_continue-2"></a>

### handle_continue/2 ###

`handle_continue(X1, Pool) -> any()`

<a name="handle_info-2"></a>

### handle_info/2 ###

<pre><code>
handle_info(Info::term(), Pool::term()) -&gt; {noreply, term()}
</code></pre>
<br />

<a name="init-1"></a>

### init/1 ###

`init(Pool) -> any()`

<a name="manual_start-0"></a>

### manual_start/0 ###

`manual_start() -> any()`

<a name="new_pool-1"></a>

### new_pool/1 ###

<pre><code>
new_pool(PoolConfig::<a href="#type-pool_config">pool_config()</a>) -&gt; {ok, pid()} | {error, {already_started, pid()}}
</code></pre>
<br />

Start a new pool described by the proplist `PoolConfig`. The
following keys are required in the proplist:



<dt><code>name</code></dt>




<dd>An atom giving the name of the pool.</dd>




<dt><code>init_count</code></dt>




<dd>Number of members to add to the pool at start. When the pool is
started, <code>init_count</code> members will be started in parallel.</dd>




<dt><code>max_count</code></dt>




<dd>Maximum number of members in the pool.</dd>




<dt><code>start_mfa</code></dt>




<dd>A tuple of the form <code>{Mod, Fun, Args}</code> describing how to start
new pool members.</dd>



In addition, you can specify any of the following optional
configuration options:



<dt><code>group</code></dt>




<dd>An atom giving the name of the group this pool belongs
to. Pools sharing a common <code>group</code> value can be accessed using
<a docgen-rel="seemfa" docgen-href="#take_group_member/1" href="#take_group_member-1"><code>take_group_member/1</code></a> and <a docgen-rel="seemfa" docgen-href="#return_group_member/2" href="#return_group_member-2"><code>return_group_member/2</code></a>.</dd>




<dt><code>cull_interval</code></dt>




<dd>Time between checks for stale pool members. Specified as
<code>{Time, Unit}</code> where <code>Time</code> is a non-negative integer and <code>Unit</code> is
one of <code>min</code>, <code>sec</code>, <code>ms</code>, or <code>mu</code>. The default value of <code>{1, min}</code>
triggers a once per minute check to remove members that have not
been accessed in <code>max_age</code> time units. Culling can be disabled by
specifying a zero time vaule (e.g. <code>{0, min}</code>. Culling will also be
disabled if <code>init_count</code> is the same as <code>max_count</code>.</dd>




<dt><code>max_age</code></dt>




<dd>Members idle longer than <code>max_age</code> time units are removed from
the pool when stale checking is enabled via
<code>cull_interval</code>. Culling of idle members will never reduce the pool
below <code>init_count</code>. The value is specified as <code>{Time, Unit}</code>. Note
that timers are not set on individual pool members and may remain
in the pool beyond the configured <code>max_age</code> value since members are
only removed on the interval configured via <code>cull_interval</code>. The
default value is <code>{30, sec}</code>.</dd>




<dt><code>member_start_timeout</code></dt>




<dd>Time limit for member starts. Specified as <code>{Time,
Unit}</code>. Defaults to <code>{1, min}</code>.</dd>



<a name="pool_child_spec-1"></a>

### pool_child_spec/1 ###

<pre><code>
pool_child_spec(PoolConfig::<a href="#type-pool_config">pool_config()</a>) -&gt; <a href="/home/seriy/workspace/stdlib/doc/supervisor.md#type-child_spec">supervisor:child_spec()</a>
</code></pre>
<br />

Get child spec described by the proplist `PoolConfig`.

See [`pooler:new_pool/1`](pooler.md#new_pool-1) for info about `PoolConfig`.

<a name="pool_stats-1"></a>

### pool_stats/1 ###

<pre><code>
pool_stats(PoolName::<a href="#type-pool_name">pool_name()</a> | pid()) -&gt; [{pid(), {reference(), free | pid(), <a href="/home/seriy/workspace/erts/doc/erlang.md#type-timestamp">erlang:timestamp()</a>}}]
</code></pre>
<br />

Obtain runtime state info for all workers.

Format of the return value is subject to change.

<a name="pool_utilization-1"></a>

### pool_utilization/1 ###

<pre><code>
pool_utilization(PoolName::<a href="#type-pool_name">pool_name()</a> | pid()) -&gt; [{max_count, pos_integer()} | {in_use_count, non_neg_integer()} | {free_count, non_neg_integer()} | {starting_count, non_neg_integer()} | {queued_count, non_neg_integer()} | {queue_max, non_neg_integer()}]
</code></pre>
<br />

Obtain utilization info for a pool.

Format of the return value is subject to change, but for now it
will be a proplist to maintain backcompat with R16.

<a name="return_group_member-2"></a>

### return_group_member/2 ###

<pre><code>
return_group_member(GroupName::<a href="#type-group_name">group_name()</a>, MemberPid::pid() | error_no_members) -&gt; ok
</code></pre>
<br />

Return a member that was taken from the group
`GroupName`. This is a convenience function for
`return_group_member/3` with `Status` of `ok`.

<a name="return_group_member-3"></a>

### return_group_member/3 ###

<pre><code>
return_group_member(GroupName::<a href="#type-group_name">group_name()</a>, MemberPid::pid() | error_no_members, Status::ok | fail) -&gt; ok
</code></pre>
<br />

Return a member that was taken from the group `GroupName`. If
`Status` is `ok` the member is returned to the pool from which is
came. If `Status` is `fail` the member will be terminated and a new
member added to the appropriate pool.

<a name="return_member-2"></a>

### return_member/2 ###

<pre><code>
return_member(PoolName::<a href="#type-pool_name">pool_name()</a> | pid(), Pid::pid() | error_no_members) -&gt; ok
</code></pre>
<br />

Return a member to the pool so it can be reused.

<a name="return_member-3"></a>

### return_member/3 ###

<pre><code>
return_member(PoolName::<a href="#type-pool_name">pool_name()</a> | pid(), Pid::pid() | error_no_members, Status::ok | fail) -&gt; ok
</code></pre>
<br />

Return a member to the pool so it can be reused.

If `Status` is 'ok', the member is returned to the pool.  If
`Status` is 'fail', the member is destroyed and a new member is
added to the pool in its place.

<a name="rm_group-1"></a>

### rm_group/1 ###

<pre><code>
rm_group(GroupName::<a href="#type-group_name">group_name()</a>) -&gt; ok | {error, {failed_rm_pools, [atom()]}}
</code></pre>
<br />

Terminates the group and all pools in that group.

If termination of any member pool fails, `rm_group/1` returns
`{error, {failed_delete_pools, Pools}}`, where `Pools` is a list
of pools that failed to terminate.

The group is NOT terminated if any member pool did not
successfully terminate.

<a name="rm_pool-1"></a>

### rm_pool/1 ###

<pre><code>
rm_pool(PoolName::<a href="#type-pool_name">pool_name()</a>) -&gt; ok | {error, not_found | running | restarting}
</code></pre>
<br />

Terminate the named pool.

<a name="start-0"></a>

### start/0 ###

<pre><code>
start() -&gt; ok
</code></pre>
<br />

<a name="start_link-1"></a>

### start_link/1 ###

`start_link(Pool) -> any()`

<a name="stop-0"></a>

### stop/0 ###

<pre><code>
stop() -&gt; ok
</code></pre>
<br />

<a name="take_group_member-1"></a>

### take_group_member/1 ###

<pre><code>
take_group_member(GroupName::<a href="#type-group_name">group_name()</a>) -&gt; pid() | error_no_members
</code></pre>
<br />

Take a member from a randomly selected member of the group
`GroupName`. Returns `MemberPid` or `error_no_members`.  If no
members are available in the randomly chosen pool, all other pools
in the group are tried in order.

<a name="take_member-1"></a>

### take_member/1 ###

<pre><code>
take_member(PoolName::<a href="#type-pool_name">pool_name()</a> | pid()) -&gt; pid() | error_no_members
</code></pre>
<br />

Obtain exclusive access to a member from `PoolName`.

If no free members are available, 'error_no_members' is returned.

<a name="take_member-2"></a>

### take_member/2 ###

<pre><code>
take_member(PoolName::<a href="#type-pool_name">pool_name()</a> | pid(), Timeout::non_neg_integer() | <a href="#type-time_spec">time_spec()</a>) -&gt; pid() | error_no_members
</code></pre>
<br />

Obtain exclusive access to a member of 'PoolName'.

If no members are available, wait for up to Timeout milliseconds for a member
to become available. Waiting requests are served in FIFO order. If no member
is available within the specified timeout, error_no_members is returned.
`Timeout` can be either milliseconds as integer or `{duration, time_unit}`

<a name="terminate-2"></a>

### terminate/2 ###

<pre><code>
terminate(Reason::term(), State::term()) -&gt; ok
</code></pre>
<br />

