

# Module pooler_starter #
* [Description](#description)
* [Function Index](#index)
* [Function Details](#functions)

Helper gen_server to start pool members.

Copyright (c) 2012-2013 Seth Falcon

__Behaviours:__ [`gen_server`](gen_server.md).

__Authors:__ Seth Falcon ([`seth@userprimary.net`](mailto:seth@userprimary.net)).

<a name="index"></a>

## Function Index ##


<table width="100%" border="1" cellspacing="0" cellpadding="2" summary="function index"><tr><td valign="top"><a href="#code_change-3">code_change/3</a></td><td></td></tr><tr><td valign="top"><a href="#handle_call-3">handle_call/3</a></td><td></td></tr><tr><td valign="top"><a href="#handle_cast-2">handle_cast/2</a></td><td></td></tr><tr><td valign="top"><a href="#handle_info-2">handle_info/2</a></td><td></td></tr><tr><td valign="top"><a href="#init-1">init/1</a></td><td></td></tr><tr><td valign="top"><a href="#start_link-2">start_link/2</a></td><td></td></tr><tr><td valign="top"><a href="#start_member-1">start_member/1</a></td><td>Start a member for the specified <code>Pool</code>.</td></tr><tr><td valign="top"><a href="#start_member-2">start_member/2</a></td><td>Same as <a docgen-rel="seemfa" docgen-href="#start_member/1" href="#start_member-1"><code>start_member/1</code></a> except that instead of calling
<a docgen-rel="seemfa" docgen-href="pooler#accept_member/2" href="pooler.md#accept_member-2"><code>pooler:accept_member/2</code></a> a raw message is sent to <code>Parent</code> of
the form <code>{accept_member, {Ref, Member}</code>.</td></tr><tr><td valign="top"><a href="#stop-1">stop/1</a></td><td></td></tr><tr><td valign="top"><a href="#stop_member_async-1">stop_member_async/1</a></td><td>Stop a member in the pool.</td></tr><tr><td valign="top"><a href="#terminate-2">terminate/2</a></td><td></td></tr></table>


<a name="functions"></a>

## Function Details ##

<a name="code_change-3"></a>

### code_change/3 ###

<pre><code>
code_change(OldVsn::term(), State::term(), Extra::term()) -&gt; {ok, term()}
</code></pre>
<br />

<a name="handle_call-3"></a>

### handle_call/3 ###

`handle_call(Request, From, State) -> any()`

<a name="handle_cast-2"></a>

### handle_cast/2 ###

`handle_cast(Request, Starter) -> any()`

<a name="handle_info-2"></a>

### handle_info/2 ###

<pre><code>
handle_info(Info::term(), Starter::term()) -&gt; {noreply, term()}
</code></pre>
<br />

<a name="init-1"></a>

### init/1 ###

<pre><code>
init(X1::{#pool{}, pid() | atom()}) -&gt; {ok, #starter{pool = #pool{}, parent = pid() | atom(), msg = term()}, 0}
</code></pre>
<br />

<a name="start_link-2"></a>

### start_link/2 ###

`start_link(Pool, Parent) -> any()`

<a name="start_member-1"></a>

### start_member/1 ###

<pre><code>
start_member(Pool::#pool{}) -&gt; pid()
</code></pre>
<br />

Start a member for the specified `Pool`.

Member creation with this call is async. This function returns
immediately with create process' pid. When the member has been
created it is sent to the specified pool via
[`pooler:accept_member/2`](pooler.md#accept_member-2).

Each call starts a single use `pooler_starter` instance via
`pooler_starter_sup`. The instance terminates normally after
creating a single member.

<a name="start_member-2"></a>

### start_member/2 ###

`start_member(Pool, Parent) -> any()`

Same as [`start_member/1`](#start_member-1) except that instead of calling
[`pooler:accept_member/2`](pooler.md#accept_member-2) a raw message is sent to `Parent` of
the form `{accept_member, {Ref, Member}`. Where `Member` will
either be the member pid or an error term and `Ref` will be the
Pid of the starter.

This is used by the init function in the `pooler` to start the
initial set of pool members in parallel.

<a name="stop-1"></a>

### stop/1 ###

`stop(Starter) -> any()`

<a name="stop_member_async-1"></a>

### stop_member_async/1 ###

<pre><code>
stop_member_async(Pid::pid()) -&gt; ok
</code></pre>
<br />

Stop a member in the pool

<a name="terminate-2"></a>

### terminate/2 ###

<pre><code>
terminate(Reason::term(), State::term()) -&gt; ok
</code></pre>
<br />

